require("lib/lincoln")
require("common")
require("enums")
require("lib/lincoln")

Globals = {
  hasRanAlready = false,
  isWormanDone = false,
  isHaldricDone = false,
  hasCivilianWalkedAway = false,
  isCibleHit = false,
  isArcheryDone = false,
  victory = false,
  hasPoorWeepingOneRun = false,
  lowered_bridge = false,
  TutorialGetObjectFirst = true,
  EdwardZoneFirst = true,
}

---Events
function Initialize(seed)
  InitializeLincoln()

  SetDoorLockedPC(GetDoorScript(DungeonTower.Door.dungeonEntrance), true)
  SetDoorLockedPC(GetDoorScript(MiscDoor.garissonEntrance), true)

  DeactivateActors({
    GetActor("Worman"),
    GetActor("Edward"),
    GetActor("Arbaletrier_Bleu_23"),
    GetActor("Epee_Bleu_36")
  })

  LockAI(GetActor("Poor_Weeping_One"), false)

  DeactivateItems({
    GetItem("Purses_7"),
    GetItem("Money_9"),
    GetItem("Clovers_11"),
    GetItem("Arrows_3"),
    GetItem("Money_6"),
    GetItem("Purses_5"),
    GetItem("Purses_10"),
    GetItem("Clover")
  })

  local reinforcements = { -- initiallly the reinforce are outmap
    "RenfortChevalier",
	"RenfortEpee1",
	"RenfortEpee2",
	"RenfortArba1",
	"RenfortArba2",
	"RenfortArba3",
	"RenfortArba4",
	}
  for _, actor in pairs(reinforcements) do
    SetActorLocation(GetActor(actor), NoWhere())
  end

  SetPersistentProperty(GetRobin(), PersistentProperty.arrows, 0)
  SetPersistentProperty(GetRobin(), PersistentProperty.purses, 0)
end

function Timer(seconds)
  if seconds % 15 == 0 and not Globals.isArcheryDone then
    SendMessage(GetActor("Shooting_Archer"), 1)
  end

  HandleFlirtingSwordsman()
  HandlePoorWeepingOneRun()
  HandleHaldricDone()
  HandleCivilianWalkedAway()
end

function CheckVictoryCondition(seconds)
  local function isNearSherwood(actor)
    local exitNodes = { "Node_28"}
    for _, node in pairs(exitNodes) do
      if IsInside(actor, GetLocation(node)) then
        return true
      end
    end
    return false
  end

  if isNearSherwood(GetRobin()) then
    CompleteObjective(1)
    return 1
  end
end

function PostInitialize()
  AddObjective(0, true)
  StartSequence()
  SequenceDisplayPopupText(0)
  Then()
  SequenceDisplayPopupText(1)
  Then()
  SequenceDisplayPopupText(2)
  EndSequence()
  AssignPatrol(GetActor("Soldier_411"), GetPatrol("Patrol_74"))
  LockAI(GetActor("Civilian_1"))
  LockAI(GetActor("Civilian_2"))
  LockAI(GetActor("Civilian_3"))
  LockAI(GetActor("Civilian_4"))
  LockAI(GetActor("Civilian_5"))
  AssignPatrol(GetActor("Soldier_45"), GetPatrol("Patrol_3"))
  AssignPatrol(GetActor("Civilian_13"), GetPatrol("Patrol_4"))
  AssignPatrol(GetActor("Soldier_75"), GetPatrol("Patrol_5"))
  AssignPatrol(GetActor("Soldier_60"), GetPatrol("Patrol_7"))
  AssignPatrol(GetActor("Soldier_63"), GetPatrol("Patrol_8"))
  AssignPatrol(GetActor("Soldier_62"), GetPatrol("Patrol_9"))
  AssignPatrol(GetActor("Soldier_66"), GetPatrol("Patrol_10"))
  AssignPatrol(GetActor("Archer_muraille 5"), GetPatrol("Patrol_12"))
  AssignPatrol(GetActor("Soldier_89"), GetPatrol("Patrol_14"))
  AssignPatrol(GetActor("Soldier_98"), GetPatrol("Patrol_15"))
end

--Functions
function HandleFlirtingSwordsman()
  if IsActorOutOfAction(GetActor("Flirting_Swordsman")) and not Globals.hasRanAlready then
    AssignPatrol(GetActor("Flirting_Swordsman"), GetPatrol("Patrol_66"))
    AssignPatrol(GetActor("WomanCivilianPoor_2"), GetPatrol("Patrol_59"))
    SetPatrolShouldRun(GetActor("WomanCivilianPoor_2"), true)
    Globals.hasRanAlready = true
  end
end



Globals = {
  pression = false,
}
function CheckIfTroopIsDead()
  -- Vérifie si le shérif est mort et que la patrouille n'a pas encore été assignée
  if Globals.pression == false and IsActorDead(GetActor("Soldier_35")) and IsActorDead(GetActor("Soldier_33")) and IsActorDead(GetActor("Soldier_34")) then
      for name, actor in pairs(GetAllActors()) do
        if name:find("Civilianaide_") then
          Globals.pression = true -- Marque comme traité
          UnlockAI(actor)
          AssignPatrol(actor, GetPatrol("Patrol_4"))
          StartSequence()
          SequenceLockAI(actor)
          Then()
          SequenceMoveIntoBuilding(actor, GetLocation("Node_1"), MovementStyle.running)
          EndSequence()
          Activate(GetItem("Clover"))
        end
      end
  end
end

function Timer(seconds)
    CheckIfTroopIsDead()
end




--function MoveSoldiersInside(soldier)
  --AssignPatrol(soldier, GetPatrol("Patrol_49"))
  --StartSequence()
  --SequenceLockAI(soldier)
  --Then()
  --SequenceMoveIntoBuilding(soldier, GetLocation("Node_1"), MovementStyle.walking)
  --EndSequence()
--end












function HandlePoorWeepingOneRun()
  if not Globals.hasPoorWeepingOneRun and
      IsActorOutOfAction(GetActor("Epee_Bleu_25")) and
      IsActorOutOfAction(GetActor("Epee_Bleu_26")) and
      IsActorOutOfAction(GetActor("Lancier_Bleu_27")) and
      IsActorOutOfAction(GetActor("Lancier_Bleu_28")) then
    SendMessage(GetActor("Poor_Weeping_One"), 1)
  end
end

function HandleWormanDone()
  if not Globals.isWormanDone and IsBonusItemPickedUp(GetItem("Money_6")) then
    Globals.isWormanDone = true
    CompleteObjective(3)
    UpdateScrollState(GetScroll("Scroll_10"), ScrollState.collected)
  end
end

function HandleHaldricDone()
  if not Globals.isHaldricDone and IsActorDead(GetActor("Haldric")) then
    Globals.isHaldricDone = true
    CompleteObjective(4)
    UpdateScrollState(GetScroll("Scroll_11"), ScrollState.collected)
  end
end

function HandleCivilianWalkedAway()
  if not Globals.hasCivilianWalkedAway and
      (IsActorOutOfAction(GetActor("Lancier_Bleu_19")) or
        IsActorOutOfAction(GetActor("Lancier_Bleu_20")) or
        IsActorOutOfAction(GetActor("Lancier_Bleu_21"))) then
    AssignPatrol(GetActor("WomanCivilianPoor_7"), GetPatrol("Patrol_58"))
    Globals.hasCivilianWalkedAway = true
  end
end

ArchersDone = function()
  if not Globals.isArcheryDone then
    CompleteObjective(5)
    Globals.isArcheryDone = true
    UpdateScrollState(GetScroll("Scroll_1"), ScrollState.persistent)
    AssignSoldiersToOfficer()
    AssignPatrolPaths()
    HandleOfficerState()
  end
end

function AssignSoldiersToOfficer()
  local subordinates = {
    GetActor("Archer_Bleu_12"),
    GetActor("Archer_Bleu_13"),
    GetActor("Shooting_Archer"),
    GetActor("Epee_Bleu_16"),
    GetActor("Epee_Bleu_17"),
    GetActor("Epee_Bleu_18")
  }

  for _, subordinate in pairs(subordinates) do
    AddAsSquadMember(GetActor("Archer_Officer"), subordinate)
  end
end

function AssignPatrolPaths()
  local patrolPaths = {
    { actor = GetActor("Lancier_Bleu_19"),     patrol = "Patrol_51" },
    { actor = GetActor("Lancier_Bleu_20"),     patrol = "Patrol_52" },
    { actor = GetActor("Lancier_Bleu_21"),     patrol = "Patrol_53" },
    { actor = GetActor("Hallebardier_Bleu_6"), patrol = "Patrol_3" },
    { actor = GetActor("Soldier_43"), patrol = "Patrol_76" }
  }

  for _, path in pairs(patrolPaths) do
    AssignPatrol(path.actor, GetPatrol(path.patrol))
  end
end

function HandleOfficerState()
  if not IsActorOutOfAction(GetActor("Archer_Officer")) then
    DisplayPopupText(6)
    AssignPatrol(GetActor("Archer_Officer"), GetPatrol("Patrol_39"))
  else
    AssignSecondaryPaths()
  end
end

function AssignSecondaryPaths()
  local secondaryPaths = {
    { actor = GetActor("Archer_Bleu_12"), patrol = "Patrol_72" },
    { actor = GetActor("Archer_Bleu_13"), patrol = "Patrol_71" },
    { actor = GetActor("Shooting_Archer"),     patrol = "Patrol_70" },
    { actor = GetActor("Epee_Bleu_16"),   patrol = "Patrol_69" },
    { actor = GetActor("Epee_Bleu_17"),   patrol = "Patrol_67" },
    { actor = GetActor("Epee_Bleu_18"),   patrol = "Patrol_68" }
  }

  for _, path in pairs(secondaryPaths) do
    AssignPatrol(path.actor, GetPatrol(path.patrol))
  end
end

function MoveSoldiersInside(soldier)
  AssignPatrol(soldier, GetPatrol("Patrol_49"))
  StartSequence()
  SequenceLockAI(soldier)
  Then()
  SequenceMoveIntoBuilding(soldier, GetLocation("Node_1"), MovementStyle.walking)
  EndSequence()
end

SoldiersAlerted = function()
  Globals.isArcheryDone = true
  UpdateScrollState(GetScroll("Scroll_1"), ScrollState.collected)
  AssignSoldiersToOfficer()
  AssignPatrolPaths()
end

RevealWeaponRoom = function()
  if not IsPatchApplied(GetPatchScript(WeaponRoom.patch)) then
    WeaponRoom:Reveal()
    ActivateActors({GetActor("Epee_Bleu_36")})
    AssignPatrol(GetActor("Epee_Bleu_36"), GetPatrol("Patrol_54"))
  end
end

RevealDungeonTower = function()
  if not IsPatchApplied(GetPatchScript(DungeonTower.patch)) then
    DungeonTower:Reveal()
    Activate(GetItem("Purses_5"))
    Activate(GetItem("Money_6"))

    ActivateActors({GetActor("Arbaletrier_Bleu_23"), GetActor("Worman")})
    AssignPatrol(GetActor("Worman"), GetPatrol("Patrol_25"))
  end
end

RevealBedRoom = function()
  if not IsPatchApplied(GetPatchScript(BedRoom.patch)) then
    BedRoom:Reveal()
    Activate(GetItem("Purses_10"))
    UpdateScrollState(GetScroll("Scroll_13"), ScrollState.collected)
  end
end

---Actors
PoorWeepingOne_80000344 = {
  ProcessMessage = function(message, unk2, unk3)
    if message == 1 then
      AssignPatrol(GetActor("Epee_Bleu_25"), GetPatrol("Patrol_26"))
      AssignPatrol(GetActor("Epee_Bleu_26"), GetPatrol("Patrol_27"))
      AssignPatrol(GetActor("Lancier_Bleu_27"), GetPatrol("Patrol_28"))
      AssignPatrol(GetActor("Lancier_Bleu_28"), GetPatrol("Patrol_29"))
      Globals.hasPoorWeepingOneRun = true
      StartSequence()
      SequenceMove(GetActor("Poor_Weeping_One"), GetLocation("Node_8"), MovementStyle.running)
      Then()
      SequenceTurnTo(GetActor("Poor_Weeping_One"), GetLocation("Node_9"))
      Then()
      SequenceCall(function()
        AttachScrollToNPC(GetActor("Poor_Weeping_One"), GetScroll("Scroll_15"))
      end)
      EndSequence()
    end
  end
}

Acher03_80000042 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

Acher02_80000041 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

Acher01_80000040 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end,

  ProcessMessage = function(message, unk2, unk3)
    if message == 1 and not Globals.isArcheryDone then
      StartSequence()
      SequenceSendMessage(GetActor("Archer_Officer"), 1)
      Then()
      SequenceTimer(12)
      Then()
      SequenceAction(GetActor("Shooting_Archer"), ActorAction.shoot, GetActorIndex(GetActor("Cible01_2")))
      Then()
      SequenceTimer(12)
      Then()
      SequenceSendMessage(GetActor("Archer_Officer"), 2)
      EndSequence()
    end
  end
}

SergentArchers_80000043 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end,

  ProcessMessage = function(message, unk2, unk3)
    if message == 1 and not Globals.isArcheryDone then
      StartSequence()
      SequenceTurnTo(GetActor("Archer_Officer"), GetLocation("Node_2"))
      EndSequence()
    end

    if message == 2 and not Globals.isArcheryDone then
      SetPersistentProperty(GetActor("Shooting_Archer"), PersistentProperty.arrows,
        (GetPersistentProperty(GetActor("Shooting_Archer"), PersistentProperty.arrows) + 1))
      StartSequence()
      SequenceTurnTo(GetActor("Archer_Officer"), GetActorLocation(GetActor("Shooting_Archer")))
      Then()
      SequencePlayAnim(GetActor("Archer_Officer"), AnimationAction.gatheringSoldiers)
      EndSequence()
    end
  end
}

SoldatCibles01_80000044 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

SoldatCibles02_8000004a = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

SoldatCibles03_80000050 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

LancierEscalierHcour01_8000005f = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

LancierHCour02_80000060 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

BouclierXT01_800000d6 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      AssignPatrol(GetActor("Flirting_Swordsman"), GetPatrol("Patrol_66"))
      AssignPatrol(GetActor("WomanCivilianPoor_2"), GetPatrol("Patrol_59"))
    end
  end
}

---Targets
Cible1_800001bb = {
  ActivatedByArrow = function(actorSource)
    HandleCibleHit(actorSource, GetActor("Cible00_1"))
  end
}

Cible2_800001bd = {
  ActivatedByArrow = function(actorSource)
    HandleCibleHit(actorSource, GetActor("Cible01_2"))
  end
}

Cible3_800001be = {
  ActivatedByArrow = function(actorSource)
    HandleCibleHit(actorSource, GetActor("Cible02_3"))
  end
}

Cible4_800001bf = {
  ActivatedByArrow = function(actorSource)
    HandleCibleHit(actorSource, GetActor("Cible03_4"))
  end
}

function HandleCibleHit(actorSource, cibleActor)
  if not Globals.isCibleHit and IsActorPC(actorSource) then
    Globals.isCibleHit = true
    StartSequence()
    SequencePlayAnimFreeze(cibleActor, AnimationAction.targetHit)
    Then()
    SequenceTimer(10)
    Then()
    SequenceCall(ArchersDone)
    EndSequence()
  end
end

 ---mecanisme_pont_levis_800001e9 = {
 --- ActivatedBySword = function(actorSource)
    --DrawBridgeRoom:ActivateDrawBridge(GetLocation("Node_15"))
  ---  ApplyPatch(GetPatchScript(3))
   --- StartSequence()
    --SequenceCall(function()
     -- Activate(GetLocation("Node_15")) 
  --  end)
  ---  Then()
   --- SequenceTimer(20)
   --- call_reinforce()
   --- EndSequence()
  ---end
---}

PontLevisDescend = {
  ActivatedBySword = function()
    DrawBridgeRoom:ActivateDrawBridge(GetLocation("Node_15"))
    call_reinforce()
    call_reinforce_deux()
  end
}

--WayPoints
OfficierdesArchers__6___80000184 = {
  ReachPoint = function(actor)
    StartSequence()
    SequenceCall(function() MoveSoldiersInside(GetActor("Archer_Officer")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Archer_Bleu_12")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Archer_Bleu_13")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Shooting_Archer")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Epee_Bleu_16")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Epee_Bleu_17")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Epee_Bleu_18")) end)
    EndSequence()
  end
}

GirlWay01__1___800001f8 = {
  ReachPoint = function(actor)
    SetPatrolShouldRun(GetActor("Flirting_Swordsman"), true)
    AssignPatrol(GetActor("Flirting_Swordsman"), GetPatrol("Patrol_61"))
    AssignPatrol(actor, GetPatrol("Patrol_65"))
  end
}

GirlWay02__1___800001f9 = {
  ReachPoint = function(actor)
    SetPatrolShouldRun(GetActor("Flirting_Swordsman"), false)
    AssignPatrol(GetActor("Flirting_Swordsman"), GetPatrol("Patrol_60"))
    AssignPatrol(actor, GetPatrol("Patrol_64"))
    StareActor(GetActor("Flirting_Swordsman"), GetActorScript(MiscAnimation.riverPier))
  end
}

--Scolls
ParchArcherDebut_80000258 = {
  Taken = function(actorSource)
    if not Globals.isArcheryDone then
      AddObjective(5, false)
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceScrollCameraTo(GetActorLocation(GetActor("Archer_Officer")))
      Then()
      SequenceTimer(15)
      Then()
      SequenceSpeak(GetActor("Archer_Officer"), 61)
      Then()
      SequenceDisplayPopupText(5)
      Then()
      SequenceDisplayPopupText(22)
      Then()
      SequenceSendMessage(GetActor("Shooting_Archer"), 1)
      Then()
      SequenceFreezeAll(false)
      SequenceUnLockUser()
      EndSequence()
    end
  end
}

ParchArgent_8000047f = {
  Taken = function(actorSource)
    if Globals.isArcheryDone then
      DisplayPopupText(19)
      Activate(GetItem("Money_9"))
      SetScrollState(ThisScroll(), ScrollState.collected)
    end
  end
}

ParchEdwardSpeak_8000025a = {
  Taken = function(actorSource)
    AddObjective(1, true)
    CompleteObjective(0)
    Reveal(GetActor("ManCivilianFriend_5"))
    StartSequence()
    SequenceSetZoomLevel(ZoomLevel.default)
    Then()
    SequenceLockUser()
    -- SequenceFreezeAll(true)
    Then()
    SequenceDisplayPopupText(8)
    Then()
    SequenceDisplayPopupText(9)
    Then()
    SequenceDisplayPopupText(10)
    Then()
    SequenceScrollCameraTo(GetLocation("Node_10"))
    Then()
    SequenceMove(GetActor("ManCivilianFriend_5"), GetLocation("Node_10"), MovementStyle.walking)
    Then()
    SequenceCall(function()
      AssignPost(GetActor("ManCivilianFriend_5"), GetLocation("Node_10"), ActorDirection.south)
      AttachScrollToNPC(GetActor("ManCivilianFriend_5"), GetScroll("Scroll_4"))
    end)
    Then()
    SequenceTimer(40)
    Then()
    SequenceJumpCameraTo(GetActorLocation(GetRobin()))
    SequenceFreezeAll(false)
    SequenceUnLockUser()
    EndSequence()
    RemoveScrollFromNPC(GetActor("Edward"))
    SetScrollState(ThisScroll(), ScrollState.collected)
  end
}

ParchFilsEdward_8000025b = {
  Taken = function(actorSource)
    AddObjective(2, true)
    CompleteObjective(1)
    DisplayPopupText(11)
    Globals.victory = true
    RemoveScrollFromNPC(GetActor("ManCivilianFriend_5"))
    SetScrollState(ThisScroll(), ScrollState.collected)
  end
}

TutGrimper_8000025d = {
  Taken = function(actorSource)
    StartSequence()
    SequenceSetZoomLevel(ZoomLevel.default)
    Then()
    SequenceLockUser()
    Then()
    SequenceFreezeAll(true)
    Then()
    SequenceScrollCameraTo(GetLocation("Node_12"))
    Then()
    SequenceTimer(12)
    Then()
    SequenceDisplayPopupText(12)
    Then()
    SequenceTimer(12)
    Then()
    SequenceFreezeAll(false)
    SequenceUnLockUser()
    EndSequence()
  end
}

TutJump_8000025e = {
  Taken = function(actorSource)
    DisplayPopupText(13)
  end
}

TutKO_8000025f = {
  Taken = function(actorSource)
    DisplayPopupText(14)
    DisplayPopupText(21)
  end
}

TutBeggar_80000261 = {
  Taken = function(actorSource)
    StartSequence()
    SequenceSetZoomLevel(1.0)
    Then()
    SequenceLockUser()
    Then()
    SequenceScrollCameraTo(GetLocation("Node_1"))
    Then()
    SequenceTimer(15)
    Then()
    SequenceReveal(GetActor("Mendicant_6"))
    Then()
    SequenceDisplayPopupText(16)
    Then()
    SequenceTimer(40)
    SequenceJumpCameraTo(GetActorLocation(actorSource))
    Then()
    SequenceUnLockUser()
    EndSequence()
  end
}

TutGetObject_80000262 = {
  Taken = function(actorSource)
    DisplayPopupText(15)
    if Globals.TutorialGetObjectFirst then
      Globals.TutorialGetObjectFirst = false
      DisplayPopupText(17)
      Activate(GetItem("Purses_7"))
    end
  end
}

BeggarWorman_80000277 = {
  Taken = function(actorSource)
    if not IsBonusItemPickedUp(GetItem("Money_6")) then
      AddObjective(3, false)
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceScrollCameraTo(GetLocation("Node_4"))
      Then()
      SequenceCall(RevealDungeonTower)
      Then()
      SequenceReveal(GetActor("Worman"))
      SequenceReveal(GetActor("Arbaletrier_Bleu_23"))
      Then()
      SequenceTimer(15)
      Then()
      SequenceDisplayPopupText(4)
      Then()
      SequenceTimer(40)
      Then()
      SequenceJumpCameraTo(GetActorLocation(actorSource))
      Then()
      SequenceUnLockUser()
      SequenceFreezeAll(false)
      EndSequence()
      SetScrollState(ThisScroll(), ScrollState.collected)
    end
  end
}

BeggarHaldric_80000278 = {
  Taken = function(actorSource)
    if GetPersistentProperty(GetActor("Haldric"), PersistentProperty.money) > 0 then
      AddObjective(4, false)
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceDisplayPopupText(7)
      Then()
      SequenceScrollCameraTo(GetActorLocation(GetActor("Haldric")))
      Then()
      SequenceTimer(15)
      Then()
      SequenceReveal(GetActor("Haldric"))
      Then()
      SequenceTimer(40)
      Then()
      SequenceJumpCameraTo(GetActorLocation(actorSource))
      Then()
      SequenceFreezeAll(false)
      SequenceUnLockUser()
      EndSequence()
      SetScrollState(ThisScroll(), ScrollState.collected)
    end
  end
}

Beggarpatch01_800002c4 = {
  Taken = function(actorSource)
    DrawBridgeRoom:Reveal()
    StartSequence()
    SequenceSetZoomLevel(ZoomLevel.default)
    Then()
    SequenceLockUser()
    SequenceFreezeAll(true)
    Then()
    SequenceScrollCameraTo(GetLocation("Node_14"))
    Then()
    SequenceTimer(15)
    Then()
    SequenceDisplayPopupText(20)
    Then()
    SequenceTimer(40)
    Then()
    SequenceJumpCameraTo(GetActorLocation(actorSource))
    Then()
    SequenceFreezeAll(false)
    SequenceUnLockUser()
    EndSequence()
  end
}

TutOpenSalle_8000046d = {
  Taken = function(actorSource)
    if not IsPatchApplied(GetPatchScript(7)) then
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceScrollCameraTo(GetLocation("Node_13"))
      Then()
      SequenceTimer(15)
      Then()
      SequenceCall(RevealBedRoom)
      Then()
      SequenceTimer(40)
      Then()
      SequenceJumpCameraTo(GetActorLocation(actorSource))
      Then()
      SequenceFreezeAll(false)
      SequenceUnLockUser()
      EndSequence()
    end
  end
}

ArrowsAtEnd_800006e3 = {
  Taken = function(actorSource)
    Activate(GetItem("Arrows_3"))
    SetScrollState(ThisScroll(), ScrollState.collected)
  end
}

PoorManScroll_80000349 = {
  Taken = function(actorSource)
    Activate(GetItem("Clovers_11"))
    RemoveScrollFromNPC(GetActor("Poor_Weeping_One"))
    SetScrollState(ThisScroll(), ScrollState.collected)
    StartSequence()
    SequenceSpeak(GetActor("Poor_Weeping_One"), 86)
    Then()
    SequenceDisplayPopupText(3)
    EndSequence()
  end
}

--Testentree = {
  --Taken = function(actorSource)
  --  AssignPatrol(GetActor("Soldier_36"), GetPatrol("Patrol_3"))
  --end
--}
---Zones
ZP06Sud_80000170 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealWeaponRoom()
    end
  end
}

ZP06Est_80000171 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealWeaponRoom()
    end
  end
}

ZP03TourOuest_80000173 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealDungeonTower()
    end
  end
}

ZP01TourPtLevis_80000175 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if not IsPatchApplied(GetPatchScript(DrawBridgeRoom.patch)) then
        DrawBridgeRoom:Reveal()
        SetScrollState(GetScroll("Scroll_12"), ScrollState.collected)
      end
    end
  end
}

ZP04CentralBas_80000177 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      Battlement:Reveal()
    end
  end
}

ZP07Bas_80000178 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealBedRoom()
    end
  end
}

ZP07Haut_80000179 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealBedRoom()
    end
  end
}

ZP05GdSalle_8000017a = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if not IsPatchApplied(GetPatchScript(StairCase.patch)) then
        StairCase:Reveal()
      end
      if not IsPatchApplied(GetPatchScript(ThroneRoom.patch)) then
        ThroneRoom:Reveal()

        Activate(GetActor("Edward"))
        AttachScrollToNPC(GetActor("Edward"), GetScroll("Scroll_3"))
        Reveal(GetActor("Edward"))
        UnlockAI(GetActor("Edward"))
      end
    end
  end
}

ZP09EscalierBas_8000017b = {
  Enter = function(actor)
    if IsActorPC(actor) and not IsPatchApplied(GetPatchScript(StairCase.patch)) then
      StairCase:Reveal()
    end
  end
}

ZP09EscalierHaut_8000017c = {
  Enter = function(actor)
    if IsActorPC(actor) and not IsPatchApplied(GetPatchScript(StairCase.patch)) then
      StairCase:Reveal()
    end
  end
}

EdwardZone_80000275 = {
  Enter = function(actor)
    if IsActorPC(actor) and Globals.EdwardZoneFirst then
      StopActor(actor)
      Globals.EdwardZoneFirst = false
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceScrollCameraTo(GetLocation("Node_5"))
      Then()
      SequenceTimer(15)
      Then()
      SequenceDisplayPopupText(18)
      Then()
      SequenceTimer(40)
      Then()
      SequenceUnLockUser()
      SequenceFreezeAll(false)
      EndSequence()
    end
  end
}

persecutionZone_80000346 = {
  Exit = function(actor)
    if GetNumberOfActorsInSector(GetLocation("Node_27")) == 1 then
      if not Globals.hasPoorWeepingOneRun then
        SendMessage(GetActor("Poor_Weeping_One"), 1)
      end
    end
  end
}

----------------------------------------

function ReinforceToOfficer() -- function that assign units Reinforce to Reinforce Officer 
  local subordinates = {
    GetActor("RenfortEpee1"),
    GetActor("RenfortEpee2"),
	GetActor("RenfortArba1"),
    GetActor("RenfortArba2"),
	GetActor("RenfortArba3"),
    GetActor("RenfortArba4")
  }

  for _, subordinate in pairs(subordinates) do
    AddAsSquadMember(GetActor("RenfortChevalier"), subordinate)
  end
end

function call_reinforce() -- Activate and unlock reinforce
	
	local reinforcements = {
    "RenfortChevalier",
	"RenfortEpee1",
	"RenfortEpee2",
	"RenfortArba1",
	"RenfortArba2",
	"RenfortArba3",
	"RenfortArba4"
	}

	for _, actor in pairs(reinforcements) do
		Activate(GetActor(actor))
		UnlockAI(GetActor(actor))
	end
	ReinforceToOfficer()	-- assign group
	AssignPatrol(GetActor("RenfortChevalier"), GetPatrol("Patrol_3")) -- assign path
end

function ReinforceToOfficerDeux() -- function that assign units Reinforce to Reinforce Officer 
  local subordinates = {
    GetActor("Soldier_68"),
    GetActor("Soldier_69"),
	GetActor("Soldier_65"),
    GetActor("Soldier_66"),
	GetActor("Soldier_67")
  }

  for _, subordinate in pairs(subordinates) do
    AddAsSquadMember(GetActor("Soldier_64"), subordinate)
  end
end

function call_reinforce_deux() -- Activate and unlock reinforce
	
	local reinforcements_deux = {
    "Soldier_64",
	"Soldier_68",
	"Soldier_69",
	"Soldier_65",
	"Soldier_66",
	"Soldier_67"
	}

	--for _, actor in pairs(reinforcements_deux) do
		--Activate(GetActor(actor))
	--	UnlockAI(GetActor(actor))
	--end
	ReinforceToOfficerDeux()	-- assign group
	AssignPatrol(GetActor("Soldier_64"), GetPatrol("Patrol_11")) -- assign path
end

Soldatsvenez = {
  Taken = function(actorSource)
    StartSequence()
    AssignPost(GetActor("Soldier_85"), GetLocation("Node_31"))
    AssignPatrol(GetActor("Soldier_85"), GetPatrol("allie1"))

    AssignPost(GetActor("Soldier_87"), GetLocation("Node_32"))
    AssignPatrol(GetActor("Soldier_87"), GetPatrol("allie2"))

    AssignPost(GetActor("Soldier_88"), GetLocation("Node_33"))
    AssignPatrol(GetActor("Soldier_88"), GetPatrol("Patrol_74"))

    AssignPost(GetActor("Soldier_91"), GetLocation("Node_34"))
    AssignPatrol(GetActor("Soldier_91"), GetPatrol("Patrol_17"))

    AssignPost(GetActor("Soldier_90"), GetLocation("Node_35"))
    AssignPatrol(GetActor("Soldier_90"), GetPatrol("Patrol_18"))

    AssignPost(GetActor("Soldier_891"), GetLocation("Node_36"))
    AssignPatrol(GetActor("Soldier_89"), GetPatrol("Patrol_19"))

    AssignPost(GetActor("Soldier_92"), GetLocation("Node_37"))
    AssignPatrol(GetActor("Soldier_92"), GetPatrol("Patrol_20"))

    EndSequence()
    SetScrollState(ThisScroll(), ScrollState.collected)
  end
}